import "Turbine.Gameplay";
import "Turbine.UI";
import "PengorosPlugins.BuffBars";
import "PengorosPlugins.UI";

ListWindow = class(Turbine.UI.Lotro.Window);

ListWindow.HighlightColor = Turbine.UI.Color(.92, 40/255, 67/255, 122/255);
ListWindow.DefaultColor = Turbine.UI.Color(.92, 0, 0, 0);
ListWindow.ItemColor = Turbine.UI.Color(245/255, 222/255, 147/255);

function ListWindow:Constructor()
    Turbine.UI.Lotro.Window.Constructor(self);
    
    self:SetVisible(false);
    self:SetSize(300, 400);

    self.removeButton = Turbine.UI.Lotro.Button();
    self.removeButton:SetParent(self);
    self.removeButton:SetPosition(45, 370);
    self.removeButton:SetSize(100, 22);
    self.removeButton:SetText(L["Remove"]);
    self.removeButton:SetEnabled(false);
    self.removeButton.Click = function(sender, args)
        self.listBox:RemoveItemAt(self.selection);
        self.selection = nil;
        self.removeButton:SetEnabled(false);
    end

    self.closeButton  = Turbine.UI.Lotro.Button();
    self.closeButton:SetParent(self);
    self.closeButton:SetPosition(155, 370);
    self.closeButton:SetSize(100, 22);
    self.closeButton:SetText(L["Close"]);
    self.closeButton.Click = function(sender, args)
        self:Close();
    end

    self.scrollBar = Turbine.UI.Lotro.ScrollBar();
    self.scrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.scrollBar:SetParent(self);
    self.scrollBar:SetSize(10, 320);
    self.scrollBar:SetPosition(283, 45);

    self.listBox = Turbine.UI.ListBox();
    self.listBox:SetParent(self);
    self.listBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self.listBox:SetVerticalScrollBar(self.scrollBar);
    self.listBox:SetMaxItemsPerLine(1);
    self.listBox:SetSize(276, 320);
    self.listBox:SetPosition(7, 45);
    self.listBox.SelectedIndexChanged = function(sender, args)
        self:ItemSelected(self.listBox:GetSelectedIndex());
    end
end

function ListWindow:ItemSelected(index)
    if (self.selection ~= nil) then
        local old = self.listBox:GetItem(self.selection);
        old:SetBackColor(ListWindow.DefaultColor);
    end
    
    local item = self.listBox:GetItem(index);
    self.selection = index;
    item:SetBackColor(ListWindow.HighlightColor);
    
    self.removeButton:SetEnabled(true);
end

function ListWindow:SetOptions(settings)
    -- clear any existing items
    local count = self.listBox:GetItemCount();
    while (count > 0) do
        local item = self.listBox:GetItem(1);
        item:SetVisible(false);
        self.listBox:RemoveItemAt(1);
        count = count - 1;
    end
    self.removeButton:SetEnabled(false);
    self.selection = nil;


    -- sort the ignored list by name
    local ordered = {};    
    for key in pairs(self.GetList(self, settings)) do
        table.insert(ordered, key);
    end
    table.sort(ordered)
    
    local width, height = self.listBox:GetSize();
    for key, val in pairs(ordered) do
        local listItem = PengorosPlugins.UI.Label();
        listItem:SetSize(width, 20);
        listItem:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
        listItem:SetForeColor(ListWindow.ItemColor);
        listItem:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
        listItem:SetText(val);
        
        listItem.MouseEnter = function(sender, args)
            sender:SetBackColor(ListWindow.HighlightColor);
        end
        listItem.MouseLeave = function(sender, args)
            if (self.listBox:IndexOfItem(sender) == self.selection) then
                sender:SetBackColor(ListWindow.HighlightColor);
            else
                sender:SetBackColor(ListWindow.DefaultColor);
            end
        end
        
        self.listBox:AddItem(listItem);
    end
end

function ListWindow:SaveOptions(settings)
    self.ResetList(self, settings);
    for i = 1, self.listBox:GetItemCount() do
        local item = self.listBox:GetItem(i);
        self.AddItemToList(self, settings, item:GetText());
    end
end
