import "Turbine.UI";
import "Turbine.UI.Lotro";
import "PengorosPlugins.Utils";
import "PengorosPlugins.BuffBars";

-- load the plugin settings
Settings.Load();

-- plugin windows
local effectWindows = {};
local effectSliders = {};
local quickslotBars = {};
local effectLogger = nil;

-- reset windows
function ResetState()
    local i;
    -- destroy the existing effect windows
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    effectWindows = nil;
    
    -- destroy the existing effect sliders
    for i = 1, table.getn(effectSliders) do
        effectSliders[i]:Destroy();
        effectSliders[i] = nil;
    end
    effectSliders = nil;

    -- destroy existing quick slots
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    quickslotBars = nil;
    
    -- destroy the existing logger
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end

    -- create new effect windows    
    effectWindows = {};
    for i = 1, table.getn(Settings.instance.windows) do
        if (Settings.instance.windows[i].enabled) then
            table.insert(effectWindows, EffectWindow(Settings.instance.windows[i]));
        end
    end

    -- create new effect sliders    
    effectSliders = {};
    for i = 1, table.getn(Settings.instance.sliders) do
        if (Settings.instance.sliders[i].enabled) then
            table.insert(effectSliders, SliderWindow(Settings.instance.sliders[i]));
        end
    end

    -- create new quickslot bars    
    quickslotBars = {};
    for i = 1, table.getn(Settings.instance.quickslots) do
        if (Settings.instance.quickslots[i].enabled) then
            table.insert(quickslotBars, QuickslotBar(Settings.instance.quickslots[i]));
        end
    end

    -- effect logger
    if (Settings.instance.log.enabled) then
        effectLogger = Log(Settings.instance.log);
    end
end

-- logging
local function EnableLogging(enable)
    if (Settings.instance.log.enabled ~= enable) then
        Settings.instance.log.enabled = enable;
        if (enable) then
            Turbine.Shell.WriteLine(L["Effect Logging Enabled"]);
            effectLogger = Log(Settings.instance.log);
        else
            Turbine.Shell.WriteLine(L["Effect Logging Disabled"]);
            effectLogger:Destroy();
            effectLogger = nil;
        end
    end
end

-- commands
buffBarsCommand = Turbine.ShellCommand();
Turbine.Shell.AddCommand("buffbars", buffBarsCommand);
function buffBarsCommand:GetShortHelp()
    return L["usage: /buffbars options | log <enable|disable>"];
end
function buffBarsCommand:GetHelp()
    return self:GetShortHelp();
end
function buffBarsCommand:Execute(cmd, args)
    if (args == L["options"] or args == L["setup"]) then
        OptionsWindow.ShowOptions();
    elseif (args == L["log enable"]) then
        EnableLogging(true);
    elseif (args == L["log disable"]) then
        EnableLogging(false);
    else
        Turbine.Shell.WriteLine(self:GetShortHelp());
    end
end

-- plugin unload event
local function UnloadPlugin()
    local i;
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    for i = 1, table.getn(effectSliders) do
        effectSliders[i]:Destroy();
        effectSliders[i] = nil;
    end
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end
    OptionsWindow.CloseOptions();
    Settings.Save();
end

-- register the unload
PengorosPlugins.Utils.KeyManager.RegisterUnloadEvent("BuffBars", UnloadPlugin);

-- set the initial state
ResetState();

Turbine.Shell.WriteLine("BuffBars v1.2.2 by Pengoros");
