import "Turbine.UI";

OptionsEffectPanel = class(Turbine.UI.ListBox);

OptionsEffectPanel.FlashElapsed = 1;
OptionsEffectPanel.FlashRemaining = 2;
OptionsEffectPanel.DiseaseElapsed = 3;
OptionsEffectPanel.DiseaseRemaining = 4;
OptionsEffectPanel.FearElapsed = 5;
OptionsEffectPanel.FearRemaining = 6;
OptionsEffectPanel.PoisonElapsed = 7;
OptionsEffectPanel.PoisonRemaining = 8;
OptionsEffectPanel.WoundElapsed = 9;
OptionsEffectPanel.WoundRemaining = 10;
OptionsEffectPanel.CorruptionElapsed = 11;
OptionsEffectPanel.CorruptionRemaining = 12;
OptionsEffectPanel.CryElapsed = 13;
OptionsEffectPanel.CryRemaining = 14;
OptionsEffectPanel.ElementalElapsed = 15;
OptionsEffectPanel.ElementalRemaining = 16;
OptionsEffectPanel.PhysicalElapsed = 17;
OptionsEffectPanel.PhysicalRemaining = 18;
OptionsEffectPanel.SongElapsed = 19;
OptionsEffectPanel.SongRemaining = 20;
OptionsEffectPanel.UndefinedElapsed = 21;
OptionsEffectPanel.UndefinedRemaining = 22;
OptionsEffectPanel.MoraleCoolDownElapsed = 23;
OptionsEffectPanel.MoraleCoolDownRemaining = 24;
OptionsEffectPanel.PowerCoolDownElapsed = 25;
OptionsEffectPanel.PowerCoolDownRemaining = 26;
OptionsEffectPanel.DiseaseCoolDownElapsed = 27;
OptionsEffectPanel.DiseaseCoolDownRemaining = 28;
OptionsEffectPanel.FearCoolDownElapsed = 29;
OptionsEffectPanel.FearCoolDownRemaining = 30;
OptionsEffectPanel.PoisonCoolDownElapsed = 31;
OptionsEffectPanel.PoisonCoolDownRemaining = 32;
OptionsEffectPanel.WoundCoolDownElapsed = 33;
OptionsEffectPanel.WoundCoolDownRemaining = 34;

-- helper functions
local tt = function(width, height, control, text)
    local tooltip = PengorosPlugins.UI.Tooltip();
    tooltip:SetText(text);
    tooltip:SetControl(control);
    tooltip:SetSize(width, height);
end

local holder = function(parent, height)
    local container = Turbine.UI.Control();
    container:SetSize(400, height);
    parent:AddItem(container);
    return container;
end

function OptionsEffectPanel:Constructor()
    Turbine.UI.ListBox.Constructor(self);

    self.current = nil;
    self.ignoreWasVisible = false;
    self.overrideWasVisible = false;
    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);
    
    -- listen for events
    PengorosPlugins.Utils.KeyManager.AddListener(self);

    local line = holder(self, 35);
    self.windowTitle = PengorosPlugins.UI.PanelDivider();
    self.windowTitle:SetParent(line);
    self.windowTitle:SetText(L["Effect Windows"]);
    self.windowTitle:SetSize(400, 30);
    
    line = holder(self, 30);
    self.windowLabel = PengorosPlugins.UI.Label();
    self.windowLabel:SetParent(line);
    self.windowLabel:SetSize(176, 20);
    self.windowLabel:SetPosition(20, 0);
    self.windowLabel:SetText(L["Window"]);
    tt(420, 19, self.windowLabel, L["Select the Effect Window you would like to modify."]);
    
    self.windowCombo = PengorosPlugins.UI.ComboBox();
    self.windowCombo:SetParent(line);
    self.windowCombo:SetSize(204, 20);
    self.windowCombo:SetPosition(196, 0);
    self.windowCombo.SelectedIndexChanged = function(sender, args)
        if (self.current ~= nil) then
            self:SaveWindowOptions(self.current);
        end
        self:SetWindowOptions(args.selection);
        self.current = args.selection;
    end
    
    line = holder(self, 30);
    self.newButton  = Turbine.UI.Lotro.Button();
    self.newButton:SetParent(line);
    self.newButton:SetPosition(89, 0);
    self.newButton:SetSize(100, 22);
    self.newButton:SetText(L["New"]);
    tt(240, 19, self.newButton, L["Create a new Effect Window."]);
    self.newButton.Click = function(sender, args)
        self:AddNewWindow();
    end

    self.copyButton  = Turbine.UI.Lotro.Button();
    self.copyButton:SetParent(line);
    self.copyButton:SetPosition(196, 0);
    self.copyButton:SetSize(100, 22);
    self.copyButton:SetText(L["Copy"]);
    tt(275, 19, self.copyButton, L["Copy the current Effect Window."]);
    self.copyButton.Click = function(sender, args)
        self:CopyCurrentWindow();
    end

    self.removeButton  = Turbine.UI.Lotro.Button();
    self.removeButton:SetParent(line);
    self.removeButton:SetPosition(303, 0);
    self.removeButton:SetSize(100, 22);
    self.removeButton:SetText(L["Remove"]);
    tt(295, 19, self.removeButton, L["Remove the current Effect Window."]);
    self.removeButton.Click = function(sender, args)
        self:RemoveCurrentWindow();
    end

    line = holder(self, 35);
    self.generalTitle = PengorosPlugins.UI.PanelDivider();
    self.generalTitle:SetParent(line);
    self.generalTitle:SetText(L["General"]);
    self.generalTitle:SetSize(400, 30);
    
    line = holder(self, 30);
    self.nameLabel = PengorosPlugins.UI.Label();
    self.nameLabel:SetParent(line);
    self.nameLabel:SetSize(100, 20);
    self.nameLabel:SetPosition(20, 0);
    self.nameLabel:SetText(L["Name"]);
    tt(420, 19, self.nameLabel, L["The descriptive name of the current Effect Window."]);
    
    self.nameText = Turbine.UI.Lotro.TextBox();
    self.nameText:SetParent(line);
    self.nameText:SetSize(280, 20);
    self.nameText:SetPosition(120, 0);
    self.nameText:SetFont(Turbine.UI.Lotro.Font.Verdana14);
    self.nameText:SetMultiline(false);
    self.nameText:SetWantsUpdates(true);
    self.nameText.Update = function(sender, args)
        self.windowCombo:UpdateSelectionText(self.nameText:GetText());
    end
    
    line = holder(self, 20);
    self.showWindowCB = PengorosPlugins.UI.CheckBox();
    self.showWindowCB:SetParent(line);
    self.showWindowCB:SetText(L["Enabled"]);
    self.showWindowCB:SetSize(400, 20);
    tt(405, 19, self.showWindowCB, L["When checked, this Effect Window will be enbled."]);

    line = holder(self, 20);
    self.showDebuffBarCB = PengorosPlugins.UI.CheckBox();
    self.showDebuffBarCB:SetParent(line);
    self.showDebuffBarCB:SetText(L["Debuff Bar"]);
    self.showDebuffBarCB:SetSize(400, 20);
    tt(455, 19, self.showDebuffBarCB, L["When checked, debuff effects will appear in the window."]);
    
    line = holder(self, 20);
    self.showBuffBarCB = PengorosPlugins.UI.CheckBox();
    self.showBuffBarCB:SetParent(line);
    self.showBuffBarCB:SetText(L["Buff Bar"]);
    self.showBuffBarCB:SetSize(400, 20);
    tt(440, 19, self.showBuffBarCB, L["When checked, buff effects will appear in the window."]);

    line = holder(self, 25);
    self.showCoolDownBarCB = PengorosPlugins.UI.CheckBox();
    self.showCoolDownBarCB:SetParent(line);
    self.showCoolDownBarCB:SetText(L["Cool Down Bar"]);
    self.showCoolDownBarCB:SetSize(400, 20);
    tt(480, 38, self.showCoolDownBarCB, L["When checked, cooldown timers will appear in the window."]);

    line = holder(self, 30);
    self.visibleTypeLabel = PengorosPlugins.UI.Label();
    self.visibleTypeLabel:SetParent(line);
    self.visibleTypeLabel:SetSize(190, 20);
    self.visibleTypeLabel:SetPosition(20, 0);
    self.visibleTypeLabel:SetText(L["Show Window"]);
    tt(285, 19, self.visibleTypeLabel, L["Select when the window is visible."]);
    
    self.visibleTypeCombo = PengorosPlugins.UI.ComboBox();
    self.visibleTypeCombo:SetParent(line);
    self.visibleTypeCombo:SetSize(190, 20);
    self.visibleTypeCombo:SetPosition(210, 0);
    self.visibleTypeCombo:AddItem(L["Always"], WindowVisibility.Always);
    self.visibleTypeCombo:AddItem(L["Only In Combat"], WindowVisibility.InCombat);
    self.visibleTypeCombo:AddItem(L["Only Out Of Combat"], WindowVisibility.OutOfCombat);

    line = holder(self, 35);
    self.sortingTitle = PengorosPlugins.UI.PanelDivider();
    self.sortingTitle:SetParent(line);
    self.sortingTitle:SetText(L["Sorting"]);
    self.sortingTitle:SetSize(400, 30);
    
    line = holder(self, 30);
    self.sortLabel = PengorosPlugins.UI.Label();
    self.sortLabel:SetParent(line);
    self.sortLabel:SetSize(190, 20);
    self.sortLabel:SetPosition(20, 0);
    self.sortLabel:SetText(L["Method"]);
    tt(250, 19, self.sortLabel, L["Select how effects are sorted."]);
    
    self.sortCombo = PengorosPlugins.UI.ComboBox();
    self.sortCombo:SetParent(line);
    self.sortCombo:SetSize(190, 20);
    self.sortCombo:SetPosition(210, 0);
    self.sortCombo:AddItem(L["Name"], EffectBarSort.Name);
    self.sortCombo:AddItem(L["Time Remaining"], EffectBarSort.Time);
    self.sortCombo:AddItem(L["Effect Category"], EffectBarSort.Category);
    self.sortCombo:AddItem(L["None"], EffectBarSort.None);
    
    line = holder(self, 30);
    self.directionLabel = PengorosPlugins.UI.Label();
    self.directionLabel:SetParent(line);
    self.directionLabel:SetSize(190, 20);
    self.directionLabel:SetPosition(20, 0);
    self.directionLabel:SetText(L["Direction"]);
    tt(345, 19, self.directionLabel, L["Select which direction effects are sorted."]);
    
    self.directionCombo = PengorosPlugins.UI.ComboBox();
    self.directionCombo:SetParent(line);
    self.directionCombo:SetSize(190, 20);
    self.directionCombo:SetPosition(210, 0);
    self.directionCombo:AddItem(L["Ascending"], true);
    self.directionCombo:AddItem(L["Descending"], false);
    
    line = holder(self, 30);
    self.orderLabel = PengorosPlugins.UI.Label();
    self.orderLabel:SetParent(line);
    self.orderLabel:SetSize(190, 20);
    self.orderLabel:SetPosition(20, 0);
    self.orderLabel:SetText(L["Bar Order"]);
    tt(360, 19, self.orderLabel, L["Select the order of the bars in the window."]);
    
    self.orderCombo = PengorosPlugins.UI.ComboBox();
    self.orderCombo:SetParent(line);
    self.orderCombo:SetSize(190, 20);
    self.orderCombo:SetPosition(210, 0);
    self.orderCombo:AddItem(L["Debuffs, Buffs, CD"], EffectBarOrder.DebuffBuffCoolDown);
    self.orderCombo:AddItem(L["Debuffs, CD, Buffs"], EffectBarOrder.DebuffCoolDownBuff);
    self.orderCombo:AddItem(L["CD, Debuffs, Buffs"], EffectBarOrder.CoolDownDebuffBuff);
    self.orderCombo:AddItem(L["CD, Buffs, Debuffs"], EffectBarOrder.CoolDownBuffDebuff);
    self.orderCombo:AddItem(L["Buffs, CD, Debuffs"], EffectBarOrder.BuffCoolDownDebuff);
    self.orderCombo:AddItem(L["Buffs, Debuffs, CD"], EffectBarOrder.BuffDebuffCoolDown);

    line = holder(self, 30);
    self.fillLabel = PengorosPlugins.UI.Label();
    self.fillLabel:SetParent(line);
    self.fillLabel:SetSize(190, 20);
    self.fillLabel:SetPosition(20, 0);
    self.fillLabel:SetText(L["Fill"]);
    tt(480, 57, self.fillLabel, L["The window will dynamically resize itself according to the number of effects active.  Fill determines which way the window grows, down or up."]);
    
    self.fillCombo = PengorosPlugins.UI.ComboBox();
    self.fillCombo:SetParent(line);
    self.fillCombo:SetSize(190, 20);
    self.fillCombo:SetPosition(210, 0);
    self.fillCombo:AddItem(L["Top to Bottom"], false);
    self.fillCombo:AddItem(L["Bottom to Top"], true);

    line = holder(self, 35);
    self.filterTitle = PengorosPlugins.UI.PanelDivider();
    self.filterTitle:SetParent(line);
    self.filterTitle:SetText(L["Effect Filters"]);
    self.filterTitle:SetSize(400, 30);
    
    line = holder(self, 20);
    self.showCurableOnlyCB = PengorosPlugins.UI.CheckBox();
    self.showCurableOnlyCB:SetParent(line);
    self.showCurableOnlyCB:SetText(L["Curable Only"]);
    self.showCurableOnlyCB:SetSize(400, 20);
    tt(410, 19, self.showCurableOnlyCB, L["When checked, only curable effects will be shown."]);

    line = holder(self, 20);
    self.showDiseaseCB = PengorosPlugins.UI.CheckBox();
    self.showDiseaseCB:SetParent(line);
    self.showDiseaseCB:SetText(L["Disease"]);
    self.showDiseaseCB:SetSize(400, 20);
    tt(360, 19, self.showDiseaseCB, L["When checked, disease effects will be shown."]);

    line = holder(self, 20);
    self.showFearCB = PengorosPlugins.UI.CheckBox();
    self.showFearCB:SetParent(line);
    self.showFearCB:SetText(L["Fear"]);
    self.showFearCB:SetSize(400, 20);
    tt(340, 19, self.showFearCB, L["When checked, fear effects will be shown."]);

    line = holder(self, 20);
    self.showPoisonCB = PengorosPlugins.UI.CheckBox();
    self.showPoisonCB:SetParent(line);
    self.showPoisonCB:SetText(L["Poison"]);
    self.showPoisonCB:SetSize(400, 20);
    tt(365, 19, self.showPoisonCB, L["When checked, poison effects will be shown."]);

    line = holder(self, 20);
    self.showWoundCB = PengorosPlugins.UI.CheckBox();
    self.showWoundCB:SetParent(line);
    self.showWoundCB:SetText(L["Wound"]);
    self.showWoundCB:SetSize(400, 20);
    tt(365, 19, self.showWoundCB, L["When checked, wound effects will be shown."]);

    line = holder(self, 20);
    self.showCorruptionCB = PengorosPlugins.UI.CheckBox();
    self.showCorruptionCB:SetParent(line);
    self.showCorruptionCB:SetText(L["Corruption"]);
    self.showCorruptionCB:SetSize(400, 20);
    tt(400, 19, self.showCorruptionCB, L["When checked, corruption effects will be shown."]);

    line = holder(self, 20);
    self.showCryCB = PengorosPlugins.UI.CheckBox();
    self.showCryCB:SetParent(line);
    self.showCryCB:SetText(L["Cry"]);
    self.showCryCB:SetSize(400, 20);
    tt(335, 19, self.showCryCB, L["When checked, cry effects will be shown."]);

    line = holder(self, 20);
    self.showElementalCB = PengorosPlugins.UI.CheckBox();
    self.showElementalCB:SetParent(line);
    self.showElementalCB:SetText(L["Elemental"]);
    self.showElementalCB:SetSize(400, 20);
    tt(385, 19, self.showElementalCB, L["When checked, elemental effects will be shown."]);

    line = holder(self, 20);
    self.showPhysicalCB = PengorosPlugins.UI.CheckBox();
    self.showPhysicalCB:SetParent(line);
    self.showPhysicalCB:SetText(L["Physical"]);
    self.showPhysicalCB:SetSize(400, 20);
    tt(370, 19, self.showPhysicalCB, L["When checked, physical effects will be shown."]);

    line = holder(self, 20);
    self.showSongCB = PengorosPlugins.UI.CheckBox();
    self.showSongCB:SetParent(line);
    self.showSongCB:SetText(L["Song"]);
    self.showSongCB:SetSize(400, 20);
    tt(350, 19, self.showSongCB, L["When checked, song effects will be shown."]);

    line = holder(self, 30);
    self.showUndefinedCB = PengorosPlugins.UI.CheckBox();
    self.showUndefinedCB:SetParent(line);
    self.showUndefinedCB:SetText(L["Undefined"]);
    self.showUndefinedCB:SetSize(400, 20);
    tt(390, 19, self.showUndefinedCB, L["When checked, undefined effects will be shown."]);

    line = holder(self, 50);
    self.minDurationSlider = PengorosPlugins.UI.Slider();
    self.minDurationSlider:SetParent(line);
    self.minDurationSlider:SetText(L["Minimum Duration (seconds)"]);
    self.minDurationSlider:SetPosition(49, 0);
    self.minDurationSlider:SetSize(350, 40);
    self.minDurationSlider:SetStep(1);
    self.minDurationSlider:SetMin(0);
    self.minDurationSlider:SetMax(3600);
    tt(480, 38, self.minDurationSlider.label, L["Effects shorter than this will not be shown.  Set to 0 to disable."]);

    line = holder(self, 50);
    self.maxDurationSlider = PengorosPlugins.UI.Slider();
    self.maxDurationSlider:SetParent(line);
    self.maxDurationSlider:SetText(L["Maximum Duration (seconds)"]);
    self.maxDurationSlider:SetPosition(49, 0);
    self.maxDurationSlider:SetSize(350, 40);
    self.maxDurationSlider:SetStep(1);
    self.maxDurationSlider:SetMin(0);
    self.maxDurationSlider:SetMax(3600);
    tt(480, 38, self.maxDurationSlider.label, L["Effects longer than this will not be shown.  Set to 0 to disable."]);

    line = holder(self, 30);
    self.ignoreWindow = IgnoreWindow();
    self.ignoreButton  = Turbine.UI.Lotro.Button();
    self.ignoreButton:SetParent(line);
    self.ignoreButton:SetPosition(76, 0);
    self.ignoreButton:SetSize(160, 22);
    self.ignoreButton:SetText(L["Ignored Effects"]);
    tt(465, 19, self.ignoreButton, L["Click the button to see a list of effects you have ignored."]);
    self.ignoreButton.Click = function(sender, args)
        self.ignoreWindow:SetVisible(true);
    end

    self.overrideWindow = OverrideWindow();
    self.overrideButton  = Turbine.UI.Lotro.Button();
    self.overrideButton:SetParent(line);
    self.overrideButton:SetPosition(243, 0);
    self.overrideButton:SetSize(160, 22);
    self.overrideButton:SetText(L["Always Shown Effects"]);
    tt(480, 38, self.overrideButton, L["Click the button to see a list of effects that will always be shown."]);
    self.overrideButton.Click = function(sender, args)
        self.overrideWindow:SetVisible(true);
    end

    line = holder(self, 35);
    self.cdFilterTitle = PengorosPlugins.UI.PanelDivider();
    self.cdFilterTitle:SetParent(line);
    self.cdFilterTitle:SetText(L["Cool Down Filters"]);
    self.cdFilterTitle:SetSize(400, 30);
    
    line = holder(self, 20);
    self.showDiseaseCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showDiseaseCoolDownCB:SetParent(line);
    self.showDiseaseCoolDownCB:SetText(L["Disease"]);
    self.showDiseaseCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showDiseaseCoolDownCB, L["When checked, disease pot cooldowns will be shown."]);

    line = holder(self, 20);
    self.showFearCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showFearCoolDownCB:SetParent(line);
    self.showFearCoolDownCB:SetText(L["Fear"]);
    self.showFearCoolDownCB:SetSize(400, 20);
    tt(415, 19, self.showFearCoolDownCB, L["When checked, fear pot cooldowns will be shown."]);

    line = holder(self, 20);
    self.showPoisonCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showPoisonCoolDownCB:SetParent(line);
    self.showPoisonCoolDownCB:SetText(L["Poison"]);
    self.showPoisonCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showPoisonCoolDownCB, L["When checked, poison pot cooldowns will be shown."]);

    line = holder(self, 20);
    self.showWoundCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showWoundCoolDownCB:SetParent(line);
    self.showWoundCoolDownCB:SetText(L["Wound"]);
    self.showWoundCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showWoundCoolDownCB, L["When checked, wound pot cooldowns will be shown."]);

    line = holder(self, 20);
    self.showMoraleCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showMoraleCoolDownCB:SetParent(line);
    self.showMoraleCoolDownCB:SetText(L["Morale"]);
    self.showMoraleCoolDownCB:SetSize(400, 20);
    tt(435, 19, self.showMoraleCoolDownCB, L["When checked, morale pot cooldowns will be shown."]);

    line = holder(self, 30);
    self.showPowerCoolDownCB = PengorosPlugins.UI.CheckBox();
    self.showPowerCoolDownCB:SetParent(line);
    self.showPowerCoolDownCB:SetText(L["Power"]);
    self.showPowerCoolDownCB:SetSize(400, 20);
    tt(430, 19, self.showPowerCoolDownCB, L["When checked, power pot cooldowns will be shown."]);

    line = holder(self, 35);
    self.flashingTitle = PengorosPlugins.UI.PanelDivider();
    self.flashingTitle:SetParent(line);
    self.flashingTitle:SetText(L["Flashing"]);
    self.flashingTitle:SetSize(400, 30);

    line = holder(self, 30);
    self.allowFlashingCB = PengorosPlugins.UI.CheckBox();
    self.allowFlashingCB:SetParent(line);
    self.allowFlashingCB:SetText(L["Flash Expiring Effects"]);
    self.allowFlashingCB:SetSize(400, 20);
    tt(480, 38, self.allowFlashingCB, L["When checked, effects will flash when they are about to expire."]);
    self.allowFlashingCB.SetEnabled = function(sender, enabled)
        PengorosPlugins.UI.CheckBox.SetEnabled(sender, enabled);
        self.flashSlider:SetEnabled(enabled and sender:IsChecked());
    end
    self.allowFlashingCB.CheckedChanged = function(sender, args)
        self.flashSlider:SetEnabled(sender:IsChecked());
    end

    line = holder(self, 50);
    self.flashSlider = PengorosPlugins.UI.Slider();
    self.flashSlider:SetParent(line);
    self.flashSlider:SetText(L["Flash Time"]);
    self.flashSlider:SetPosition(49, 0);
    self.flashSlider:SetSize(350, 40);
    self.flashSlider:SetStep(1);
    self.flashSlider:SetMin(1);
    self.flashSlider:SetMax(30);
    tt(480, 57, self.flashSlider.label, L["Adjust the time when effects start to flash.  Effects start flashing slowly and increase speed when the time remaining is half of this value."]);

    line = holder(self, 35);
    self.uiTitle = PengorosPlugins.UI.PanelDivider();
    self.uiTitle:SetParent(line);
    self.uiTitle:SetText(L["UI Elements"]);
    self.uiTitle:SetSize(400, 30);

    line = holder(self, 20);
    self.showEffectIconCB = PengorosPlugins.UI.CheckBox();
    self.showEffectIconCB:SetParent(line);
    self.showEffectIconCB:SetText(L["Effect Icon"]);
    self.showEffectIconCB:SetSize(400, 20);
    tt(480, 38, self.showEffectIconCB, L["When checked, the icon for the effect will be placed on the bar."]);

    line = holder(self, 20);
    self.showEffectNameCB = PengorosPlugins.UI.CheckBox();
    self.showEffectNameCB:SetParent(line);
    self.showEffectNameCB:SetText(L["Effect Name"]);
    self.showEffectNameCB:SetSize(400, 20);
    tt(480, 38, self.showEffectNameCB, L["When checked, the name for the effect will be placed on the bar."]);

    line = holder(self, 20);
    self.showEffectTimerCB = PengorosPlugins.UI.CheckBox();
    self.showEffectTimerCB:SetParent(line);
    self.showEffectTimerCB:SetText(L["Effect Timer"]);
    self.showEffectTimerCB:SetSize(400, 20);
    tt(480, 38, self.showEffectTimerCB, L["When checked, the timer for the effect will be placed on the bar."]);

    line = holder(self, 25);
    self.showDurationTimerCB = PengorosPlugins.UI.CheckBox();
    self.showDurationTimerCB:SetParent(line);
    self.showDurationTimerCB:SetText(L["Timer On Effects With No Duration"]);
    self.showDurationTimerCB:SetSize(400, 20);
    tt(480, 57, self.showDurationTimerCB, L["When checked, effects that do not have a duration will have an incrementing timer placed on the bar indicating how long the effect has been in place."]);

    line = holder(self, 30);
    self.formatLabel = PengorosPlugins.UI.Label();
    self.formatLabel:SetParent(line);
    self.formatLabel:SetSize(190, 20);
    self.formatLabel:SetPosition(20, 0);
    self.formatLabel:SetText(L["Timer Format"]);
    tt(330, 19, self.formatLabel, L["Select the display format for the timers."]);
    
    self.formatCombo = PengorosPlugins.UI.ComboBox();
    self.formatCombo:SetParent(line);
    self.formatCombo:SetSize(190, 20);
    self.formatCombo:SetPosition(210, 0);
    self.formatCombo:AddItem(L["Clock: "]..PengorosPlugins.Utils.TimeFormat.CLOCK(75), TimeDisplayFormat.Clock);
    self.formatCombo:AddItem(L["Truncated: "]..PengorosPlugins.Utils.TimeFormat.TRUNCATED(75), TimeDisplayFormat.Truncated);
    self.formatCombo:AddItem(L["Literal: "]..PengorosPlugins.Utils.TimeFormat.LITERAL(75), TimeDisplayFormat.Literal);

    line = holder(self, 30);
    self.fontLabel = PengorosPlugins.UI.Label();
    self.fontLabel:SetParent(line);
    self.fontLabel:SetSize(190, 20);
    self.fontLabel:SetPosition(20, 0);
    self.fontLabel:SetText(L["Font Size"]);
    tt(325, 19, self.fontLabel, L["Select the font size to use for the text."]);
    
    self.fontCombo = PengorosPlugins.UI.FontComboBox();
    self.fontCombo:SetParent(line);
    self.fontCombo:SetSize(190, 20);
    self.fontCombo:SetPosition(210, 0);
    self.fontCombo:AddItem(L["23px tall"], Turbine.UI.Lotro.Font.Verdana23, 27);
    self.fontCombo:AddItem(L["22px tall"], Turbine.UI.Lotro.Font.Verdana22, 26);
    self.fontCombo:AddItem(L["20px tall"], Turbine.UI.Lotro.Font.Verdana20, 24);
    self.fontCombo:AddItem(L["18px tall"], Turbine.UI.Lotro.Font.Verdana18, 22);
    self.fontCombo:AddItem(L["16px tall"], Turbine.UI.Lotro.Font.Verdana16, 20);
    self.fontCombo:AddItem(L["14px tall"], Turbine.UI.Lotro.Font.Verdana14, 18);
    self.fontCombo:AddItem(L["12px tall"], Turbine.UI.Lotro.Font.Verdana12, 16);
    self.fontCombo:AddItem(L["10px tall"], Turbine.UI.Lotro.Font.Verdana10, 14);

    line = holder(self, 50);
    self.widthSlider = PengorosPlugins.UI.Slider();
    self.widthSlider:SetParent(line);
    self.widthSlider:SetText(L["Width"]);
    self.widthSlider:SetPosition(49, 0);
    self.widthSlider:SetSize(350, 40);
    self.widthSlider:SetStep(1);
    self.widthSlider:SetMin(50);
    self.widthSlider:SetMax(500);
    tt(275, 19, self.widthSlider.label, L["Adjust the width of the window."]);

    line = holder(self, 50);
    self.heightSlider = PengorosPlugins.UI.Slider();
    self.heightSlider:SetParent(line);
    self.heightSlider:SetText(L["Height"]);
    self.heightSlider:SetPosition(49, 0);
    self.heightSlider:SetSize(350, 40);
    self.heightSlider:SetStep(1);
    self.heightSlider:SetMin(10);
    self.heightSlider:SetMax(50);
    tt(280, 19, self.heightSlider.label, L["Adjust the height of the window."]);

    line = holder(self, 35);
    self.colorTitle = PengorosPlugins.UI.PanelDivider();
    self.colorTitle:SetParent(line);
    self.colorTitle:SetText(L["Color"]);
    self.colorTitle:SetSize(400, 30);

    line = holder(self, 30);
    self.background = Turbine.UI.Control();
    self.background:SetParent(line);
    self.background:SetPosition(60, 0);
    self.background:SetSize(90, 20);
    self.background:SetZOrder(10);
    self.background:SetBackColor(Turbine.UI.Color(.5,0,0,0));

    self.color = Turbine.UI.Control();
    self.color:SetParent(line);
    self.color:SetPosition(60, 0);
    self.color:SetSize(90, 20);
    self.color:SetZOrder(20);
    self.color:SetBackColor(Turbine.UI.Color(1,.8,.8,0));

    self.colorCombo = PengorosPlugins.UI.ComboBox();
    self.colorCombo:SetParent(line);
    self.colorCombo:SetSize(220, 20);
    self.colorCombo:SetPosition(180, 0);
    tt(230, 19, self.colorCombo, L["Select the color to modify."]);
    self.colorCombo:AddItem(L["Flash Elapsed"], OptionsEffectPanel.FlashElapsed);
    self.colorCombo:AddItem(L["Flash Remaining"], OptionsEffectPanel.FlashRemaining);
    self.colorCombo:AddItem(L["Disease Elapsed"], OptionsEffectPanel.DiseaseElapsed);
    self.colorCombo:AddItem(L["Disease Remaining"], OptionsEffectPanel.DiseaseRemaining);
    self.colorCombo:AddItem(L["Fear Elapsed"], OptionsEffectPanel.FearElapsed);
    self.colorCombo:AddItem(L["Fear Remaining"], OptionsEffectPanel.FearRemaining);
    self.colorCombo:AddItem(L["Poison Elapsed"], OptionsEffectPanel.PoisonElapsed);
    self.colorCombo:AddItem(L["Poison Remaining"], OptionsEffectPanel.PoisonRemaining);
    self.colorCombo:AddItem(L["Wound Elapsed"], OptionsEffectPanel.WoundElapsed);
    self.colorCombo:AddItem(L["Wound Remaining"], OptionsEffectPanel.WoundRemaining);
    self.colorCombo:AddItem(L["Corruption Elapsed"], OptionsEffectPanel.CorruptionElapsed);
    self.colorCombo:AddItem(L["Corruption Remaining"], OptionsEffectPanel.CorruptionRemaining);
    self.colorCombo:AddItem(L["Cry Elapsed"], OptionsEffectPanel.CryElapsed);
    self.colorCombo:AddItem(L["Cry Remaining"], OptionsEffectPanel.CryRemaining);
    self.colorCombo:AddItem(L["Elemental Elapsed"], OptionsEffectPanel.ElementalElapsed);
    self.colorCombo:AddItem(L["Elemental Remaining"], OptionsEffectPanel.ElementalRemaining);
    self.colorCombo:AddItem(L["Physical Elapsed"], OptionsEffectPanel.PhysicalElapsed);
    self.colorCombo:AddItem(L["Physical Remaining"], OptionsEffectPanel.PhysicalRemaining);
    self.colorCombo:AddItem(L["Song Elapsed"], OptionsEffectPanel.SongElapsed);
    self.colorCombo:AddItem(L["Song Remaining"], OptionsEffectPanel.SongRemaining);
    self.colorCombo:AddItem(L["Undefined Elapsed"], OptionsEffectPanel.UndefinedElapsed);
    self.colorCombo:AddItem(L["Undefined Remaining"], OptionsEffectPanel.UndefinedRemaining);
    self.colorCombo:AddItem(L["Morale CD Elapsed"], OptionsEffectPanel.MoraleCoolDownElapsed);
    self.colorCombo:AddItem(L["Morale CD Remaining"], OptionsEffectPanel.MoraleCoolDownRemaining);
    self.colorCombo:AddItem(L["Power CD Elapsed"], OptionsEffectPanel.PowerCoolDownElapsed);
    self.colorCombo:AddItem(L["Power CD Remaining"], OptionsEffectPanel.PowerCoolDownRemaining);
    self.colorCombo:AddItem(L["Disease CD Elapsed"], OptionsEffectPanel.DiseaseCoolDownElapsed);
    self.colorCombo:AddItem(L["Disease CD Remaining"], OptionsEffectPanel.DiseaseCoolDownRemaining);
    self.colorCombo:AddItem(L["Fear CD Elapsed"], OptionsEffectPanel.FearCoolDownElapsed);
    self.colorCombo:AddItem(L["Fear CD Remaining"], OptionsEffectPanel.FearCoolDownRemaining);
    self.colorCombo:AddItem(L["Poison CD Elapsed"], OptionsEffectPanel.PoisonCoolDownElapsed);
    self.colorCombo:AddItem(L["Poison CD Remaining"], OptionsEffectPanel.PoisonCoolDownRemaining);
    self.colorCombo:AddItem(L["Wound CD Elapsed"], OptionsEffectPanel.WoundCoolDownElapsed);
    self.colorCombo:AddItem(L["Wound CD Remaining"], OptionsEffectPanel.WoundCoolDownRemaining);
    self.colorCombo.SelectedIndexChanged = function(sender, args)
        self:ChangeColor(self.colors[args.selection]);
    end    

    line = holder(self, 50);
    self.redSlider = PengorosPlugins.UI.Slider();
    self.redSlider:SetParent(line);
    self.redSlider:SetText(L["Red"]);
    self.redSlider:SetPosition(49, 0);
    self.redSlider:SetSize(350, 40);
    self.redSlider:SetStep(1);
    self.redSlider:SetMin(0);
    self.redSlider:SetMax(255);
    tt(350, 19, self.redSlider.label, L["Adjust the red value of the selected color."]);
    self.redSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.r = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.greenSlider = PengorosPlugins.UI.Slider();
    self.greenSlider:SetParent(line);
    self.greenSlider:SetText(L["Green"]);
    self.greenSlider:SetPosition(49, 0);
    self.greenSlider:SetSize(350, 40);
    self.greenSlider:SetStep(1);
    self.greenSlider:SetMin(0);
    self.greenSlider:SetMax(255);
    tt(370, 19, self.greenSlider.label, L["Adjust the green value of the selected color."]);
    self.greenSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.g = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.blueSlider = PengorosPlugins.UI.Slider();
    self.blueSlider:SetParent(line);
    self.blueSlider:SetText(L["Blue"]);
    self.blueSlider:SetPosition(49, 0);
    self.blueSlider:SetSize(350, 40);
    self.blueSlider:SetStep(1);
    self.blueSlider:SetMin(0);
    self.blueSlider:SetMax(255);
    tt(355, 19, self.blueSlider.label, L["Adjust the blue value of the selected color."]);
    self.blueSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.b = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end

    line = holder(self, 50);
    self.alphaSlider = PengorosPlugins.UI.Slider();
    self.alphaSlider:SetParent(line);
    self.alphaSlider:SetText(L["Alpha"]);
    self.alphaSlider:SetPosition(49, 0);
    self.alphaSlider:SetSize(350, 40);
    self.alphaSlider:SetStep(1);
    self.alphaSlider:SetMin(0);
    self.alphaSlider:SetMax(255);
    tt(480, 38, self.alphaSlider.label, L["Adjust the alpha value of the selected color.  0 is fully transparent and 255 is fully opaque."]);
    self.alphaSlider.ValueChanged = function(sender, args)
        local color = self.colors[self.colorCombo:GetSelection()];
        color.a = args.value;
        self.color:SetBackColor(Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255));
    end
end

function OptionsEffectPanel:AddNewWindow()
    local window = EffectBarSettings();
    window.name = L["New Window"];
    self.windowCombo:AddItem(window.name, window);
    self.windowCombo:SetSelection(window);
    self:EnableControls(true);
end

function OptionsEffectPanel:CopyCurrentWindow()
    self:SaveWindowOptions(self.current); -- make sure the options are up to date 
    local window = self.current:Copy();
    window.name = window.name .. " - " .. L["Copy"];
    self.windowCombo:AddItem(window.name, window);
    self.windowCombo:SetSelection(window);
    self:EnableControls(true);
end

function OptionsEffectPanel:RemoveCurrentWindow()
    self.windowCombo:RemoveItem(self.current);
    self:EnableControls(self.windowCombo:GetItemCount() > 0);
end

function OptionsEffectPanel:ChangeColor(color)
    self.color:SetBackColor(Turbine.UI.Color(color["a"] / 255, color["r"] / 255, color["g"] / 255, color["b"] / 255));
    self.redSlider:SetValue(color["r"]);
    self.greenSlider:SetValue(color["g"]);
    self.blueSlider:SetValue(color["b"]);
    self.alphaSlider:SetValue(color["a"]);
end

function OptionsEffectPanel:Closed()
    self.ignoreWindow:Close();
    self.overrideWindow:Close();
    PengorosPlugins.Utils.KeyManager.RemoveListener(self);
end

function OptionsEffectPanel:UiHidden(hidden)
    if (hidden) then
        self.ignoreWasVisible = self.ignoreWindow:IsVisible();
        self.ignoreWindow:SetVisible(false);
        self.overrideWasVisible = self.overrideWindow:IsVisible();
        self.overrideWindow:SetVisible(false);
    else
        self.ignoreWindow:SetVisible(self.ignoreWasVisible);
        self.overrideWindow:SetVisible(self.overrideWasVisible);
    end
end

function OptionsEffectPanel:EnableControls(enable)
    self.windowCombo:SetEnabled(enable);
    self.removeButton:SetEnabled(enable);
    self.copyButton:SetEnabled(enable);
    self.nameLabel:SetEnabled(enable);
    self.nameText:SetEnabled(enable);
    self.showWindowCB:SetEnabled(enable);
    self.showDebuffBarCB:SetEnabled(enable);
    self.showBuffBarCB:SetEnabled(enable);
    self.showCoolDownBarCB:SetEnabled(enable);
    self.visibleTypeLabel:SetEnabled(enable);
    self.visibleTypeCombo:SetEnabled(enable);
    self.sortLabel:SetEnabled(enable);
    self.sortCombo:SetEnabled(enable);
    self.directionLabel:SetEnabled(enable);
    self.directionCombo:SetEnabled(enable);
    self.orderLabel:SetEnabled(enable);
    self.orderCombo:SetEnabled(enable);
    self.fillLabel:SetEnabled(enable);
    self.fillCombo:SetEnabled(enable);
    self.showCurableOnlyCB:SetEnabled(enable);
    self.showDiseaseCB:SetEnabled(enable);
    self.showFearCB:SetEnabled(enable);
    self.showPoisonCB:SetEnabled(enable);
    self.showWoundCB:SetEnabled(enable);
    self.showCorruptionCB:SetEnabled(enable);
    self.showCryCB:SetEnabled(enable);
    self.showElementalCB:SetEnabled(enable);
    self.showPhysicalCB:SetEnabled(enable);
    self.showSongCB:SetEnabled(enable);
    self.showUndefinedCB:SetEnabled(enable);
    self.minDurationSlider:SetEnabled(enable);
    self.maxDurationSlider:SetEnabled(enable);
    self.showDiseaseCoolDownCB:SetEnabled(enable);
    self.showFearCoolDownCB:SetEnabled(enable);
    self.showPoisonCoolDownCB:SetEnabled(enable);
    self.showWoundCoolDownCB:SetEnabled(enable);
    self.showMoraleCoolDownCB:SetEnabled(enable);
    self.showPowerCoolDownCB:SetEnabled(enable);
    self.ignoreButton:SetEnabled(enable);
    self.overrideButton:SetEnabled(enable);
    self.allowFlashingCB:SetEnabled(enable);
    self.showEffectIconCB:SetEnabled(enable);
    self.showEffectNameCB:SetEnabled(enable);
    self.showEffectTimerCB:SetEnabled(enable);
    self.showDurationTimerCB:SetEnabled(enable);
    self.formatLabel:SetEnabled(enable);
    self.formatCombo:SetEnabled(enable);
    self.fontLabel:SetEnabled(enable);
    self.fontCombo:SetEnabled(enable);
    self.widthSlider:SetEnabled(enable);
    self.heightSlider:SetEnabled(enable);
    self.colorCombo:SetEnabled(enable);
    self.redSlider:SetEnabled(enable);
    self.greenSlider:SetEnabled(enable);
    self.blueSlider:SetEnabled(enable);
    self.alphaSlider:SetEnabled(enable);
    self.color:SetVisible(enable);
    self.background:SetVisible(enable);
end

function OptionsEffectPanel:SetOptions()
    -- windows
    local i;
    local size = table.getn(Settings.instance.windows);
    for i = 1, size do
        local window = Settings.instance.windows[i]:Copy();
        self.windowCombo:AddItem(window.name, window);
    end
    
    if (size > 0) then
        self.windowCombo:SetSelection(self.windowCombo:GetItem(1));
    else
        self:EnableControls(false);
    end
end

function OptionsEffectPanel:SaveOptions()
    -- save the current window settings
    if (self.current ~= nil) then
        self:SaveWindowOptions(self.current);
    end
    
    -- collect all the windows into an array
    local windows = {};
    local i;
    for i = 1, self.windowCombo:GetItemCount() do
        windows[i] = self.windowCombo:GetItem(i);
    end
    
    -- overwrite the existing windows
    Settings.instance.windows = windows;
end

function OptionsEffectPanel:SetWindowOptions(settings)
    -- ignore 
    self.ignoreWindow:SetOptions(settings);
    self.overrideWindow:SetOptions(settings);
    
    -- general
    self.nameText:SetText(settings.name);
    self.showWindowCB:SetChecked(settings.enabled);
    self.showDebuffBarCB:SetChecked(settings.showDebuffBar);
    self.showBuffBarCB:SetChecked(settings.showBuffBar);
    self.showCoolDownBarCB:SetChecked(settings.showCoolDownBar);
    self.visibleTypeCombo:SetSelection(settings.visibleType);

    -- sorting
    self.sortCombo:SetSelection(settings.sortMethod);
    self.directionCombo:SetSelection(settings.sortAscending);
    self.orderCombo:SetSelection(settings.barOrder);
    self.fillCombo:SetSelection(settings.fillBottomToTop);

    -- effect filters
    self.showCurableOnlyCB:SetChecked(settings.showCurableOnly);
    self.showDiseaseCB:SetChecked(settings.showDisease);
    self.showFearCB:SetChecked(settings.showFear);
    self.showWoundCB:SetChecked(settings.showWound);
    self.showPoisonCB:SetChecked(settings.showPoison);
    self.showCorruptionCB:SetChecked(settings.showCorruption);
    self.showCryCB:SetChecked(settings.showCry);
    self.showElementalCB:SetChecked(settings.showElemental);
    self.showPhysicalCB:SetChecked(settings.showPhysical);
    self.showSongCB:SetChecked(settings.showSong);
    self.showUndefinedCB:SetChecked(settings.showUndefined);
    self.minDurationSlider:SetValue(settings.minDuration);
    self.maxDurationSlider:SetValue(settings.maxDuration);
    
    -- cool down filters
    self.showDiseaseCoolDownCB:SetChecked(settings.showDiseaseCD);
    self.showFearCoolDownCB:SetChecked(settings.showFearCD);
    self.showPoisonCoolDownCB:SetChecked(settings.showPoisonCD);
    self.showWoundCoolDownCB:SetChecked(settings.showWoundCD);
    self.showMoraleCoolDownCB:SetChecked(settings.showMoraleCD);
    self.showPowerCoolDownCB:SetChecked(settings.showPowerCD);
    
    -- flahing
    self.allowFlashingCB:SetChecked(settings.allowFlashing);
    self.flashSlider:SetValue(settings.flashTime);
    
    -- ui elements
    self.showEffectIconCB:SetChecked(settings.showEffectIcon);
    self.showEffectNameCB:SetChecked(settings.showEffectName);
    self.showEffectTimerCB:SetChecked(settings.showEffectTimer);
    self.showDurationTimerCB:SetChecked(settings.showEffectDuration);
    self.formatCombo:SetSelection(settings.displayFormat);
    self.fontCombo:SetSelection(settings.font);
    self.widthSlider:SetValue(settings.barWidth);
    self.heightSlider:SetValue(settings.barHeight);
    
    -- colors
    self.colors = {};
    self.colors[OptionsEffectPanel.FlashElapsed] = settings:GetElapsedColorArray(Colors.Flash);
    self.colors[OptionsEffectPanel.FlashRemaining] = settings:GetRemainingColorArray(Colors.Flash);
    self.colors[OptionsEffectPanel.DiseaseElapsed] = settings:GetElapsedColorArray(Colors.Disease);
    self.colors[OptionsEffectPanel.DiseaseRemaining] = settings:GetRemainingColorArray(Colors.Disease);
    self.colors[OptionsEffectPanel.FearElapsed] = settings:GetElapsedColorArray(Colors.Fear);
    self.colors[OptionsEffectPanel.FearRemaining] = settings:GetRemainingColorArray(Colors.Fear);
    self.colors[OptionsEffectPanel.PoisonElapsed] = settings:GetElapsedColorArray(Colors.Poison);
    self.colors[OptionsEffectPanel.PoisonRemaining] = settings:GetRemainingColorArray(Colors.Poison);
    self.colors[OptionsEffectPanel.WoundElapsed] = settings:GetElapsedColorArray(Colors.Wound);
    self.colors[OptionsEffectPanel.WoundRemaining] = settings:GetRemainingColorArray(Colors.Wound);
    self.colors[OptionsEffectPanel.CorruptionElapsed] = settings:GetElapsedColorArray(Colors.Corruption);
    self.colors[OptionsEffectPanel.CorruptionRemaining] = settings:GetRemainingColorArray(Colors.Corruption);
    self.colors[OptionsEffectPanel.CryElapsed] = settings:GetElapsedColorArray(Colors.Cry);
    self.colors[OptionsEffectPanel.CryRemaining] = settings:GetRemainingColorArray(Colors.Cry);
    self.colors[OptionsEffectPanel.ElementalElapsed] = settings:GetElapsedColorArray(Colors.Elemental);
    self.colors[OptionsEffectPanel.ElementalRemaining] = settings:GetRemainingColorArray(Colors.Elemental);
    self.colors[OptionsEffectPanel.PhysicalElapsed] = settings:GetElapsedColorArray(Colors.Physical);
    self.colors[OptionsEffectPanel.PhysicalRemaining] = settings:GetRemainingColorArray(Colors.Physical);
    self.colors[OptionsEffectPanel.SongElapsed] = settings:GetElapsedColorArray(Colors.Song);
    self.colors[OptionsEffectPanel.SongRemaining] = settings:GetRemainingColorArray(Colors.Song);
    self.colors[OptionsEffectPanel.UndefinedElapsed] = settings:GetElapsedColorArray(Colors.Undefined);
    self.colors[OptionsEffectPanel.UndefinedRemaining] = settings:GetRemainingColorArray(Colors.Undefined);
    self.colors[OptionsEffectPanel.MoraleCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Morale);
    self.colors[OptionsEffectPanel.MoraleCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Morale);
    self.colors[OptionsEffectPanel.PowerCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Power);
    self.colors[OptionsEffectPanel.PowerCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Power);
    self.colors[OptionsEffectPanel.DiseaseCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Disease);
    self.colors[OptionsEffectPanel.DiseaseCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Disease);
    self.colors[OptionsEffectPanel.FearCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Fear);
    self.colors[OptionsEffectPanel.FearCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Fear);
    self.colors[OptionsEffectPanel.PoisonCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Poison);
    self.colors[OptionsEffectPanel.PoisonCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Poison);
    self.colors[OptionsEffectPanel.WoundCoolDownElapsed] = settings:GetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Wound);
    self.colors[OptionsEffectPanel.WoundCoolDownRemaining] = settings:GetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Wound);    
    self.background:SetBackColor(settings:GetBackColor());
    self.colorCombo:SetSelection(OptionsEffectPanel.FlashElapsed);
    self:ChangeColor(self.colors[OptionsEffectPanel.FlashElapsed]);
end

function OptionsEffectPanel:SaveWindowOptions(settings)
    -- ignore
    self.ignoreWindow:SaveOptions(settings);
    self.overrideWindow:SaveOptions(settings);

    -- general
    settings.name = self.nameText:GetText();
    settings.enabled = self.showWindowCB:IsChecked();
    settings.showDebuffBar = self.showDebuffBarCB:IsChecked();
    settings.showBuffBar = self.showBuffBarCB:IsChecked();
    settings.showCoolDownBar = self.showCoolDownBarCB:IsChecked();
    settings.visibleType = self.visibleTypeCombo:GetSelection();
   
    -- sorting
    settings.sortMethod = self.sortCombo:GetSelection();
    settings.sortAscending = self.directionCombo:GetSelection();
    settings.barOrder = self.orderCombo:GetSelection();
    settings.fillBottomToTop = self.fillCombo:GetSelection();
       
    -- effect filters
    settings.showCurableOnly = self.showCurableOnlyCB:IsChecked();
    settings.showDisease = self.showDiseaseCB:IsChecked();
    settings.showFear = self.showFearCB:IsChecked();
    settings.showWound = self.showWoundCB:IsChecked();
    settings.showPoison = self.showPoisonCB:IsChecked();
    settings.showCorruption = self.showCorruptionCB:IsChecked();
    settings.showCry = self.showCryCB:IsChecked();
    settings.showElemental = self.showElementalCB:IsChecked();
    settings.showPhysical = self.showPhysicalCB:IsChecked();
    settings.showSong = self.showSongCB:IsChecked();
    settings.showUndefined = self.showUndefinedCB:IsChecked();
    settings.minDuration = self.minDurationSlider:GetValue();
    settings.maxDuration = self.maxDurationSlider:GetValue();
    
    -- cool down filters
    settings.showDiseaseCD = self.showDiseaseCoolDownCB:IsChecked();
    settings.showFearCD = self.showFearCoolDownCB:IsChecked();
    settings.showPoisonCD = self.showPoisonCoolDownCB:IsChecked();
    settings.showWoundCD = self.showWoundCoolDownCB:IsChecked();
    settings.showMoraleCD = self.showMoraleCoolDownCB:IsChecked();
    settings.showPowerCD = self.showPowerCoolDownCB:IsChecked();
       
    -- flashing
    settings.allowFlashing = self.allowFlashingCB:IsChecked();
    settings.flashTime = self.flashSlider:GetValue();
   
    -- ui elements
    settings.showEffectIcon = self.showEffectIconCB:IsChecked();
    settings.showEffectName = self.showEffectNameCB:IsChecked();
    settings.showEffectTimer = self.showEffectTimerCB:IsChecked();
    settings.showEffectDuration = self.showDurationTimerCB:IsChecked();
    settings.displayFormat = self.formatCombo:GetSelection();
    settings.font = self.fontCombo:GetSelection();
    settings.barWidth = self.widthSlider:GetValue();
    settings.barHeight = self.heightSlider:GetValue();
   
    -- colors
    settings:SetElapsedColorArray(Colors.Flash, self.colors[OptionsEffectPanel.FlashElapsed]);
    settings:SetRemainingColorArray(Colors.Flash, self.colors[OptionsEffectPanel.FlashRemaining]);
    settings:SetElapsedColorArray(Colors.Disease, self.colors[OptionsEffectPanel.DiseaseElapsed]);
    settings:SetRemainingColorArray(Colors.Disease, self.colors[OptionsEffectPanel.DiseaseRemaining]);
    settings:SetElapsedColorArray(Colors.Fear, self.colors[OptionsEffectPanel.FearElapsed]);
    settings:SetRemainingColorArray(Colors.Fear, self.colors[OptionsEffectPanel.FearRemaining]);
    settings:SetElapsedColorArray(Colors.Poison, self.colors[OptionsEffectPanel.PoisonElapsed]);
    settings:SetRemainingColorArray(Colors.Poison, self.colors[OptionsEffectPanel.PoisonRemaining]);
    settings:SetElapsedColorArray(Colors.Wound, self.colors[OptionsEffectPanel.WoundElapsed]);
    settings:SetRemainingColorArray(Colors.Wound, self.colors[OptionsEffectPanel.WoundRemaining]);
    settings:SetElapsedColorArray(Colors.Corruption, self.colors[OptionsEffectPanel.CorruptionElapsed]);
    settings:SetRemainingColorArray(Colors.Corruption, self.colors[OptionsEffectPanel.CorruptionRemaining]);
    settings:SetElapsedColorArray(Colors.Cry, self.colors[OptionsEffectPanel.CryElapsed]);
    settings:SetRemainingColorArray(Colors.Cry, self.colors[OptionsEffectPanel.CryRemaining]);
    settings:SetElapsedColorArray(Colors.Elemental, self.colors[OptionsEffectPanel.ElementalElapsed]);
    settings:SetRemainingColorArray(Colors.Elemental, self.colors[OptionsEffectPanel.ElementalRemaining]);
    settings:SetElapsedColorArray(Colors.Physical, self.colors[OptionsEffectPanel.PhysicalElapsed]);
    settings:SetRemainingColorArray(Colors.Physical, self.colors[OptionsEffectPanel.PhysicalRemaining]);
    settings:SetElapsedColorArray(Colors.Song, self.colors[OptionsEffectPanel.SongElapsed]);
    settings:SetRemainingColorArray(Colors.Song, self.colors[OptionsEffectPanel.SongRemaining]);
    settings:SetElapsedColorArray(Colors.Undefined, self.colors[OptionsEffectPanel.UndefinedElapsed]);
    settings:SetRemainingColorArray(Colors.Undefined, self.colors[OptionsEffectPanel.UndefinedRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Morale, self.colors[OptionsEffectPanel.MoraleCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Morale, self.colors[OptionsEffectPanel.MoraleCoolDownRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Power, self.colors[OptionsEffectPanel.PowerCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Power, self.colors[OptionsEffectPanel.PowerCoolDownRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Disease, self.colors[OptionsEffectPanel.DiseaseCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Disease, self.colors[OptionsEffectPanel.DiseaseCoolDownRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Fear, self.colors[OptionsEffectPanel.FearCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Fear, self.colors[OptionsEffectPanel.FearCoolDownRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Poison, self.colors[OptionsEffectPanel.PoisonCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Poison, self.colors[OptionsEffectPanel.PoisonCoolDownRemaining]);
    settings:SetElapsedColorArray(PengorosPlugins.Utils.CoolDownCategory.Wound, self.colors[OptionsEffectPanel.WoundCoolDownElapsed]);
    settings:SetRemainingColorArray(PengorosPlugins.Utils.CoolDownCategory.Wound, self.colors[OptionsEffectPanel.WoundCoolDownRemaining]);
end
