import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

Trigger = class();

function Trigger:Constructor(values)
    self.template = values.template;
    self.triggerType = values.triggerType; -- the type of trigger, effect or level for now
    self.name = values.name; -- the name of the trigger
    
    -- effect values
    self.category = values.category; -- effect category, can be nil for all
    self.isCurable = values.isCurable; -- is the effect curable
    self.match = values.match; -- name match string
    if (self.match == "") then
        self.match = nil;
    end
    if (self.match ~= nil) then
    	self.match = self.match:lower(); -- store patterns in lowercase
    end
    self.hasEffect = values.hasEffect; -- trigger matches if the effect is present or absent

    -- level values
    self.level = values.level; -- level power or morale
    self.threshold = values.threshold; -- type specific threshold, i.e. morale percent level
    
    -- response
    self.shortcutType = values.shortcutType;
    self.shortcutData = values.shortcutData;
end

function Trigger:GetOptionName()
	if (self.shortcutData == nil or self.shortcutData == "") then
		return self.name .. " " .. L["(edit to set the shortcut)"]
	else
		return self.name;
	end
end

function Trigger:HasEffect()
    return self.hasEffect;
end

function Trigger:MatchesEffect(effect)
    if (self.category ~= nil and self.category ~= effect:GetCategory()) then
        return false;
    elseif (self.isCurable ~= nil and self.isCurable ~= effect:IsCurable()) then
        return false;
    elseif (self.match ~= nil and string.match(effect:GetName():lower(), self.match) == nil) then
        return false;
    end
    
    return true;
end

function Trigger:MatchesLevel(level)
    if (self.threshold <= level) then
        return false;
    end
    
    return true;
end

function Trigger:MatchesLevelType(levelType)
    if (self.level ~= levelType) then
        return false;
    end

    return true;
end

function Trigger:GetShortcut()
    return Turbine.UI.Lotro.Shortcut(self.shortcutType, self.shortcutData);
end
